@@ -22,10 +22,10 @@ func fieldNames() []string {
 	names := []string{}
 	var data store.FormatData
 	t := reflect.TypeOf(data)
-	for i := 0; i < t.NumField(); i++ {
+	for i := range t.NumField() {
 		f := t.Field(i)
 		if f.Anonymous {
-			for j := 0; j < f.Type.NumField(); j++ {
+			for j := range f.Type.NumField() {
 				names = append(names, f.Type.Field(j).Name)
 			}
 		} else {
@@ -39,15 +39,15 @@ func newListCommand() *cobra.Command {
 	listCommand := &cobra.Command{
 		Use:     "list [flags] [INSTANCE]...",
 		Aliases: []string{"ls"},
-		Short:   "List instances of Lima.",
+		Short:   "List instances of Lima",
 		Long: `List instances of Lima.
 
 The output can be presented in one of several formats, using the --format <format> flag.
 
-  --format json  - output in json format
-  --format yaml  - output in yaml format
-  --format table - output in table format
-  --format '{{ <go template> }}' - if the format begins and ends with '{{ }}', then it is used as a go template.
+  --format json  - Output in JSON format
+  --format yaml  - Output in YAML format
+  --format table - Output in table format
+  --format '{{ <go template> }}' - If the format begins and ends with '{{ }}', then it is used as a go template.
 ` + store.FormatHelp + `
 The following legacy flags continue to function:
   --json - equal to '--format json'`,
@@ -57,7 +57,7 @@ The following legacy flags continue to function:
 		GroupID:           basicCommand,
 	}
 
-	listCommand.Flags().StringP("format", "f", "table", "output format, one of: json, yaml, table, go-template")
+	listCommand.Flags().StringP("format", "f", "table", "Output format, one of: json, yaml, table, go-template")
 	listCommand.Flags().Bool("list-fields", false, "List fields available for format")
 	listCommand.Flags().Bool("json", false, "JSONify output")
 	listCommand.Flags().BoolP("quiet", "q", false, "Only show names")
@@ -134,11 +134,11 @@ func listAction(cmd *cobra.Command, args []string) error {
 		logrus.Warnf("The directory %q does not look like a valid Lima directory: %v", store.Directory(), err)
 	}
 
-	allinstances, err := store.Instances()
+	allInstances, err := store.Instances()
 	if err != nil {
 		return err
 	}
-	if len(allinstances) == 0 {
+	if len(args) == 0 && len(allInstances) == 0 {
 		logrus.Warn("No instance found. Run `limactl create` to create an instance.")
 		return nil
 	}
@@ -147,7 +147,7 @@ func listAction(cmd *cobra.Command, args []string) error {
 	unmatchedInstances := false
 	if len(args) > 0 {
 		for _, arg := range args {
-			matches := instanceMatches(arg, allinstances)
+			matches := instanceMatches(arg, allInstances)
 			if len(matches) > 0 {
 				instanceNames = append(instanceNames, matches...)
 			} else {
@@ -156,7 +156,7 @@ func listAction(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		instanceNames = allinstances
+		instanceNames = allInstances
 	}
 
 	if quiet {
