@@ -28,6 +28,16 @@ const (
 )
 
 func main() {
+	if runtime.GOOS == "windows" {
+		extras, hasExtra := os.LookupEnv("_LIMA_WINDOWS_EXTRA_PATH")
+		if hasExtra && strings.TrimSpace(extras) != "" {
+			p := os.Getenv("PATH")
+			err := os.Setenv("PATH", strings.TrimSpace(extras)+string(filepath.ListSeparator)+p)
+			if err != nil {
+				logrus.Warning("Can't add extras to PATH, relying entirely on system PATH")
+			}
+		}
+	}
 	if err := newApp().Execute(); err != nil {
 		handleExitCoder(err)
 		logrus.Fatal(err)
@@ -65,9 +75,10 @@ func newApp() *cobra.Command {
 	}
 	rootCmd.PersistentFlags().String("log-level", "", "Set the logging level [trace, debug, info, warn, error]")
 	rootCmd.PersistentFlags().String("log-format", "text", "Set the logging format [text, json]")
-	rootCmd.PersistentFlags().Bool("debug", false, "debug mode")
+	rootCmd.PersistentFlags().Bool("debug", false, "Debug mode")
 	// TODO: "survey" does not support using cygwin terminal on windows yet
 	rootCmd.PersistentFlags().Bool("tty", isatty.IsTerminal(os.Stdout.Fd()), "Enable TUI interactions such as opening an editor. Defaults to true when stdout is a terminal. Set to false for automation.")
+	rootCmd.PersistentFlags().BoolP("yes", "y", false, "Alias of --tty=false")
 	rootCmd.PersistentPreRunE = func(cmd *cobra.Command, _ []string) error {
 		l, _ := cmd.Flags().GetString("log-level")
 		if l != "" {
@@ -129,6 +140,22 @@ func newApp() *cobra.Command {
 		if nfs {
 			return errors.New("must not run on NFS dir")
 		}
+
+		if cmd.Flags().Changed("yes") && cmd.Flags().Changed("tty") {
+			return errors.New("cannot use both --tty and --yes flags at the same time")
+		}
+
+		if cmd.Flags().Changed("yes") {
+			// Sets the value of the yesValue flag by using the yes flag.
+			yesValue, _ := cmd.Flags().GetBool("yes")
+			if yesValue {
+				// Sets to the default value false
+				err := cmd.Flags().Set("tty", "false")
+				if err != nil {
+					return err
+				}
+			}
+		}
 		return nil
 	}
 	rootCmd.AddGroup(&cobra.Group{ID: "basic", Title: "Basic Commands:"})
@@ -142,7 +169,6 @@ func newApp() *cobra.Command {
 		newListCommand(),
 		newDeleteCommand(),
 		newValidateCommand(),
-		newSudoersCommand(),
 		newPruneCommand(),
 		newHostagentCommand(),
 		newInfoCommand(),
@@ -159,10 +185,10 @@ func newApp() *cobra.Command {
 		newUnprotectCommand(),
 		newTunnelCommand(),
 		newTemplateCommand(),
+		newRestartCommand(),
+		newSudoersCommand(),
+		newStartAtLoginCommand(),
 	)
-	if runtime.GOOS == "darwin" || runtime.GOOS == "linux" {
-		rootCmd.AddCommand(startAtLoginCommand())
-	}
 
 	return rootCmd
 }
