@@ -9,6 +9,7 @@ import (
 	_ "embed"
 	"errors"
 	"fmt"
+	"maps"
 	"net"
 	"os"
 	"os/user"
@@ -23,12 +24,14 @@ import (
 	"github.com/coreos/go-semver/semver"
 	"github.com/docker/go-units"
 	"github.com/goccy/go-yaml"
+	"github.com/lima-vm/lima/pkg/ioutilx"
 	"github.com/lima-vm/lima/pkg/version"
 	"github.com/pbnjay/memory"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/cpu"
 
 	"github.com/lima-vm/lima/pkg/identifierutil"
+	"github.com/lima-vm/lima/pkg/localpathutil"
 	. "github.com/lima-vm/lima/pkg/must"
 	"github.com/lima-vm/lima/pkg/networks"
 	"github.com/lima-vm/lima/pkg/osutil"
@@ -58,33 +61,41 @@ var (
 )
 
 func defaultCPUType() CPUType {
+	// x86_64 + TCG + max was previously unstable until 2021.
+	// https://bugzilla.redhat.com/show_bug.cgi?id=1999700
+	// https://bugs.launchpad.net/qemu/+bug/1748296
+	defaultX8664 := "max"
+	if runtime.GOOS == "windows" && runtime.GOARCH == "amd64" {
+		// https://github.com/lima-vm/lima/pull/3487#issuecomment-2846253560
+		// > #931 intentionally prevented the code from setting it to max when running on Windows,
+		// > and kept it at qemu64.
+		//
+		// TODO: remove this if "max" works with the latest qemu
+		defaultX8664 = "qemu64"
+	}
 	cpuType := map[Arch]string{
-		AARCH64: "cortex-a76", // available since QEMU 7.1 (Aug 2022)
-		ARMV7L:  "cortex-a7",
-		// Since https://github.com/lima-vm/lima/pull/494, we use qemu64 cpu for better emulation of x86_64.
-		X8664:   "qemu64",
-		RISCV64: "rv64", // FIXME: what is the right choice for riscv64?
+		AARCH64: "max",
+		ARMV7L:  "max",
+		X8664:   defaultX8664,
+		PPC64LE: "max",
+		RISCV64: "max",
+		S390X:   "max",
 	}
 	for arch := range cpuType {
 		if IsNativeArch(arch) && IsAccelOS() {
 			if HasHostCPU() {
 				cpuType[arch] = "host"
-			} else if HasMaxCPU() {
-				cpuType[arch] = "max"
 			}
 		}
 		if arch == X8664 && runtime.GOOS == "darwin" {
-			switch cpuType[arch] {
-			case "host", "max":
-				// disable AVX-512, since it requires trapping instruction faults in guest
-				// Enterprise Linux requires either v2 (SSE4) or v3 (AVX2), but not yet v4.
-				cpuType[arch] += ",-avx512vl"
+			// disable AVX-512, since it requires trapping instruction faults in guest
+			// Enterprise Linux requires either v2 (SSE4) or v3 (AVX2), but not yet v4.
+			cpuType[arch] += ",-avx512vl"
 
-				// Disable pdpe1gb on Intel Mac
-				// https://github.com/lima-vm/lima/issues/1485
-				// https://stackoverflow.com/a/72863744/5167443
-				cpuType[arch] += ",-pdpe1gb"
-			}
+			// Disable pdpe1gb on Intel Mac
+			// https://github.com/lima-vm/lima/issues/1485
+			// https://stackoverflow.com/a/72863744/5167443
+			cpuType[arch] += ",-pdpe1gb"
 		}
 	}
 	return cpuType
@@ -467,15 +478,9 @@ func FillDefault(y, d, o *LimaYAML, filePath string, warn bool) {
 
 	hosts := make(map[string]string)
 	// Values can be either names or IP addresses. Name values are canonicalized in the hostResolver.
-	for k, v := range d.HostResolver.Hosts {
-		hosts[k] = v
-	}
-	for k, v := range y.HostResolver.Hosts {
-		hosts[k] = v
-	}
-	for k, v := range o.HostResolver.Hosts {
-		hosts[k] = v
-	}
+	maps.Copy(hosts, d.HostResolver.Hosts)
+	maps.Copy(hosts, y.HostResolver.Hosts)
+	maps.Copy(hosts, o.HostResolver.Hosts)
 	y.HostResolver.Hosts = hosts
 
 	y.Provision = slices.Concat(o.Provision, y.Provision, d.Provision)
@@ -487,10 +492,47 @@ func FillDefault(y, d, o *LimaYAML, filePath string, warn bool) {
 		if provision.Mode == ProvisionModeDependency && provision.SkipDefaultDependencyResolution == nil {
 			provision.SkipDefaultDependencyResolution = ptr.Of(false)
 		}
-		if out, err := executeGuestTemplate(provision.Script, instDir, y.User, y.Param); err == nil {
-			provision.Script = out.String()
-		} else {
-			logrus.WithError(err).Warnf("Couldn't process provisioning script %q as a template", provision.Script)
+		if provision.Mode == ProvisionModeData {
+			if provision.Content == nil {
+				provision.Content = ptr.Of("")
+			} else {
+				if out, err := executeGuestTemplate(*provision.Content, instDir, y.User, y.Param); err == nil {
+					provision.Content = ptr.Of(out.String())
+				} else {
+					logrus.WithError(err).Warnf("Couldn't process data content %q as a template", *provision.Content)
+				}
+			}
+			if provision.Overwrite == nil {
+				provision.Overwrite = ptr.Of(true)
+			}
+			if provision.Owner == nil {
+				provision.Owner = ptr.Of("root:root")
+			} else {
+				if out, err := executeGuestTemplate(*provision.Owner, instDir, y.User, y.Param); err == nil {
+					provision.Owner = ptr.Of(out.String())
+				} else {
+					logrus.WithError(err).Warnf("Couldn't owner %q as a template", *provision.Owner)
+				}
+			}
+			// Path is required; validation will throw an error when it is nil
+			if provision.Path != nil {
+				if out, err := executeGuestTemplate(*provision.Path, instDir, y.User, y.Param); err == nil {
+					provision.Path = ptr.Of(out.String())
+				} else {
+					logrus.WithError(err).Warnf("Couldn't process path %q as a template", *provision.Path)
+				}
+			}
+			if provision.Permissions == nil {
+				provision.Permissions = ptr.Of("644")
+			}
+		}
+		// TODO Turn Script into a pointer; it is a plain string for historical reasons only
+		if provision.Script != "" {
+			if out, err := executeGuestTemplate(provision.Script, instDir, y.User, y.Param); err == nil {
+				provision.Script = out.String()
+			} else {
+				logrus.WithError(err).Warnf("Couldn't process provisioning script %q as a template", provision.Script)
+			}
 		}
 	}
 
@@ -703,7 +745,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string, warn bool) {
 	location := make(map[string]int)
 	for _, mount := range slices.Concat(d.Mounts, y.Mounts, o.Mounts) {
 		if out, err := executeHostTemplate(mount.Location, instDir, y.Param); err == nil {
-			mount.Location = out.String()
+			mount.Location = filepath.Clean(out.String())
 		} else {
 			logrus.WithError(err).Warnf("Couldn't process mount location %q as a template", mount.Location)
 		}
@@ -785,8 +827,21 @@ func FillDefault(y, d, o *LimaYAML, filePath string, warn bool) {
 				mounts[i].NineP.Cache = ptr.Of(Default9pCacheForRO)
 			}
 		}
+		if location, err := localpathutil.Expand(mount.Location); err == nil {
+			mounts[i].Location = location
+		} else {
+			logrus.WithError(err).Warnf("Couldn't expand location %q", mount.Location)
+		}
 		if mount.MountPoint == nil {
-			mounts[i].MountPoint = ptr.Of(mount.Location)
+			mountLocation := mounts[i].Location
+			if runtime.GOOS == "windows" {
+				var err error
+				mountLocation, err = ioutilx.WindowsSubsystemPath(mountLocation)
+				if err != nil {
+					logrus.WithError(err).Warnf("Couldn't convert location %q into mount target", mounts[i].Location)
+				}
+			}
+			mounts[i].MountPoint = ptr.Of(mountLocation)
 		}
 	}
 
@@ -799,27 +854,15 @@ func FillDefault(y, d, o *LimaYAML, filePath string, warn bool) {
 	}
 
 	env := make(map[string]string)
-	for k, v := range d.Env {
-		env[k] = v
-	}
-	for k, v := range y.Env {
-		env[k] = v
-	}
-	for k, v := range o.Env {
-		env[k] = v
-	}
+	maps.Copy(env, d.Env)
+	maps.Copy(env, y.Env)
+	maps.Copy(env, o.Env)
 	y.Env = env
 
 	param := make(map[string]string)
-	for k, v := range d.Param {
-		param[k] = v
-	}
-	for k, v := range y.Param {
-		param[k] = v
-	}
-	for k, v := range o.Param {
-		param[k] = v
-	}
+	maps.Copy(param, d.Param)
+	maps.Copy(param, y.Param)
+	maps.Copy(param, o.Param)
 	y.Param = param
 
 	if y.CACertificates.RemoveDefaults == nil {
@@ -919,14 +962,22 @@ func executeHostTemplate(format, instDir string, param map[string]string) (bytes
 	tmpl, err := template.New("").Parse(format)
 	if err == nil {
 		limaHome, _ := dirnames.LimaDir()
+		globalTempDir := "/tmp"
+		if runtime.GOOS == "windows" {
+			// On Windows the global temp directory "%SystemRoot%\Temp" (normally "C:\Windows\Temp")
+			// is not writable by regular users.
+			globalTempDir = os.TempDir()
+		}
 		data := map[string]any{
 			"Dir":  instDir,
 			"Name": filepath.Base(instDir),
 			// TODO: add hostname fields for the host and the guest
-			"UID":   currentUser.Uid,
-			"User":  currentUser.Username,
-			"Home":  userHomeDir,
-			"Param": param,
+			"UID":           currentUser.Uid,
+			"User":          currentUser.Username,
+			"Home":          userHomeDir,
+			"Param":         param,
+			"GlobalTempDir": globalTempDir,
+			"TempDir":       os.TempDir(),
 
 			"Instance": filepath.Base(instDir), // DEPRECATED, use `{{.Name}}`
 			"LimaHome": limaHome,               // DEPRECATED, use `{{.Dir}}` instead of `{{.LimaHome}}/{{.Instance}}`
@@ -1045,8 +1096,12 @@ func NewArch(arch string) Arch {
 		}
 		logrus.Warnf("Unknown arm: %d", arm)
 		return arch
+	case "ppc64le":
+		return PPC64LE
 	case "riscv64":
 		return RISCV64
+	case "s390x":
+		return S390X
 	default:
 		logrus.Warnf("Unknown arch: %s", arch)
 		return arch
@@ -1127,8 +1182,8 @@ func ResolveVMType(y, d, o *LimaYAML, filePath string) VMType {
 				logrus.Debugf("ResolveVMType: resolved VMType %q (non-native arch=%q is specified in []*LimaYAML{o,y,d}[%d])", QEMU, *f.Arch, i)
 				return QEMU
 			}
-			if f.Firmware.LegacyBIOS != nil && *f.Firmware.LegacyBIOS {
-				logrus.Debugf("ResolveVMType: resolved VMType %q (firmware.legacyBIOS is specified in []*LimaYAML{o,y,d}[%d])", QEMU, i)
+			if ResolveArch(f.Arch) == X8664 && f.Firmware.LegacyBIOS != nil && *f.Firmware.LegacyBIOS {
+				logrus.Debugf("ResolveVMType: resolved VMType %q (firmware.legacyBIOS is specified in []*LimaYAML{o,y,d}[%d], on x86_64)", QEMU, i)
 				return QEMU
 			}
 			if f.MountType != nil && *f.MountType == NINEP {
@@ -1179,7 +1234,7 @@ func ResolveArch(s *string) Arch {
 
 func IsAccelOS() bool {
 	switch runtime.GOOS {
-	case "darwin", "linux", "netbsd", "windows":
+	case "darwin", "linux", "netbsd", "windows", "dragonfly":
 		// Accelerator
 		return true
 	}
@@ -1227,17 +1282,14 @@ func HasHostCPU() bool {
 	return false
 }
 
-func HasMaxCPU() bool {
-	// windows: WHPX: Unexpected VP exit code 4
-	return HasHostCPU()
-}
-
 func IsNativeArch(arch Arch) bool {
 	nativeX8664 := arch == X8664 && runtime.GOARCH == "amd64"
 	nativeAARCH64 := arch == AARCH64 && runtime.GOARCH == "arm64"
 	nativeARMV7L := arch == ARMV7L && runtime.GOARCH == "arm" && goarm() == 7
+	nativePPC64LE := arch == PPC64LE && runtime.GOARCH == "ppc64le"
 	nativeRISCV64 := arch == RISCV64 && runtime.GOARCH == "riscv64"
-	return nativeX8664 || nativeAARCH64 || nativeARMV7L || nativeRISCV64
+	nativeS390X := arch == S390X && runtime.GOARCH == "s390x"
+	return nativeX8664 || nativeAARCH64 || nativeARMV7L || nativePPC64LE || nativeRISCV64 || nativeS390X
 }
 
 func unique(s []string) []string {
