@@ -7,6 +7,7 @@ import (
 	"fmt"
 	"net"
 	"os"
+	"path"
 	"path/filepath"
 	"runtime"
 	"slices"
@@ -15,6 +16,7 @@ import (
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
+	"github.com/lima-vm/lima/pkg/ioutilx"
 	"github.com/lima-vm/lima/pkg/osutil"
 	"github.com/lima-vm/lima/pkg/ptr"
 	"github.com/lima-vm/lima/pkg/store/dirnames"
@@ -48,8 +50,12 @@ func TestFillDefault(t *testing.T) {
 			t.Skipf("unsupported GOARM: %d", arm)
 		}
 		arch = ARMV7L
+	case "ppc64le":
+		arch = PPC64LE
 	case "riscv64":
 		arch = RISCV64
+	case "s390x":
+		arch = S390X
 	default:
 		t.Skipf("unknown GOARCH: %s", runtime.GOARCH)
 	}
@@ -59,10 +65,7 @@ func TestFillDefault(t *testing.T) {
 	limaHome, err := dirnames.LimaDir()
 	assert.NilError(t, err)
 	user := osutil.LimaUser("0.0.0", false)
-	if runtime.GOOS != "windows" {
-		// manual template expansion for "/home/{{.User}}.linux" (done by FillDefault)
-		user.HomeDir = fmt.Sprintf("/home/%s.linux", user.Username)
-	}
+	user.HomeDir = fmt.Sprintf("/home/%s.linux", user.Username)
 	uid, err := strconv.ParseUint(user.Uid, 10, 32)
 	assert.NilError(t, err)
 
@@ -146,8 +149,9 @@ func TestFillDefault(t *testing.T) {
 			},
 		},
 		Mounts: []Mount{
-			{Location: "/tmp"},
-			{Location: "{{.Dir}}/{{.Param.ONE}}", MountPoint: ptr.Of("/mnt/{{.Param.ONE}}")},
+			//nolint:usetesting // We need the OS temp directory name here; it is not used to create temp files for testing
+			{Location: filepath.Clean(os.TempDir())},
+			{Location: filepath.Clean("{{.Dir}}/{{.Param.ONE}}"), MountPoint: ptr.Of("/mnt/{{.Param.ONE}}")},
 		},
 		MountType: ptr.Of(NINEP),
 		Provision: []Provision{
@@ -221,6 +225,12 @@ func TestFillDefault(t *testing.T) {
 
 	expect.Mounts = slices.Clone(y.Mounts)
 	expect.Mounts[0].MountPoint = ptr.Of(expect.Mounts[0].Location)
+	if runtime.GOOS == "windows" {
+		mountLocation, err := ioutilx.WindowsSubsystemPath(expect.Mounts[0].Location)
+		if err == nil {
+			expect.Mounts[0].MountPoint = ptr.Of(mountLocation)
+		}
+	}
 	expect.Mounts[0].Writable = ptr.Of(false)
 	expect.Mounts[0].SSHFS.Cache = ptr.Of(true)
 	expect.Mounts[0].SSHFS.FollowSymlinks = ptr.Of(false)
@@ -231,8 +241,8 @@ func TestFillDefault(t *testing.T) {
 	expect.Mounts[0].NineP.Cache = ptr.Of(Default9pCacheForRO)
 	expect.Mounts[0].Virtiofs.QueueSize = nil
 	// Only missing Mounts field is Writable, and the default value is also the null value: false
-	expect.Mounts[1].Location = fmt.Sprintf("%s/%s", instDir, y.Param["ONE"])
-	expect.Mounts[1].MountPoint = ptr.Of(fmt.Sprintf("/mnt/%s", y.Param["ONE"]))
+	expect.Mounts[1].Location = filepath.Join(instDir, y.Param["ONE"])
+	expect.Mounts[1].MountPoint = ptr.Of(path.Join("/mnt", y.Param["ONE"]))
 	expect.Mounts[1].Writable = ptr.Of(false)
 	expect.Mounts[1].SSHFS.Cache = ptr.Of(true)
 	expect.Mounts[1].SSHFS.FollowSymlinks = ptr.Of(false)
@@ -321,6 +331,9 @@ func TestFillDefault(t *testing.T) {
 	// User-provided defaults should override any builtin defaults
 
 	// Choose values that are different from the "builtin" defaults
+
+	// Calling filepath.Abs() to add a drive letter on Windows
+	varLog, _ := filepath.Abs("/var/log")
 	d = LimaYAML{
 		VMType: ptr.Of("vz"),
 		OS:     ptr.Of("unknown"),
@@ -329,7 +342,9 @@ func TestFillDefault(t *testing.T) {
 			AARCH64: "arm64",
 			ARMV7L:  "armhf",
 			X8664:   "amd64",
+			PPC64LE: "ppc64le",
 			RISCV64: "riscv64",
+			S390X:   "s390x",
 		},
 		CPUs:   ptr.Of(7),
 		Memory: ptr.Of("5GiB"),
@@ -385,7 +400,7 @@ func TestFillDefault(t *testing.T) {
 
 		Mounts: []Mount{
 			{
-				Location: "/var/log",
+				Location: varLog,
 				Writable: ptr.Of(false),
 			},
 		},
@@ -456,6 +471,12 @@ func TestFillDefault(t *testing.T) {
 	expect.Containerd.Archives[0].Arch = *d.Arch
 	expect.Mounts = slices.Clone(d.Mounts)
 	expect.Mounts[0].MountPoint = ptr.Of(expect.Mounts[0].Location)
+	if runtime.GOOS == "windows" {
+		mountLocation, err := ioutilx.WindowsSubsystemPath(expect.Mounts[0].Location)
+		if err == nil {
+			expect.Mounts[0].MountPoint = ptr.Of(mountLocation)
+		}
+	}
 	expect.Mounts[0].SSHFS.Cache = ptr.Of(true)
 	expect.Mounts[0].SSHFS.FollowSymlinks = ptr.Of(false)
 	expect.Mounts[0].SSHFS.SFTPDriver = ptr.Of("")
@@ -541,7 +562,9 @@ func TestFillDefault(t *testing.T) {
 			AARCH64: "uber-arm",
 			ARMV7L:  "armv8",
 			X8664:   "pentium",
+			PPC64LE: "power10",
 			RISCV64: "sifive-u54",
+			S390X:   "z14",
 		},
 		CPUs:   ptr.Of(12),
 		Memory: ptr.Of("7GiB"),
@@ -593,7 +616,7 @@ func TestFillDefault(t *testing.T) {
 
 		Mounts: []Mount{
 			{
-				Location: "/var/log",
+				Location: varLog,
 				Writable: ptr.Of(true),
 				SSHFS: SSHFS{
 					Cache:          ptr.Of(false),
