@@ -13,7 +13,6 @@ import (
 	"io/fs"
 	"os"
 	"os/exec"
-	"os/user"
 	"path/filepath"
 	"regexp"
 	"runtime"
@@ -31,7 +30,6 @@ import (
 	"github.com/lima-vm/lima/pkg/fileutils"
 	"github.com/lima-vm/lima/pkg/iso9660util"
 	"github.com/lima-vm/lima/pkg/limayaml"
-	"github.com/lima-vm/lima/pkg/localpathutil"
 	"github.com/lima-vm/lima/pkg/networks"
 	"github.com/lima-vm/lima/pkg/qemu/imgutil"
 	"github.com/lima-vm/lima/pkg/store"
@@ -46,12 +44,45 @@ type Config struct {
 	LimaYAML     *limayaml.LimaYAML
 	SSHLocalPort int
 	SSHAddress   string
+	VirtioGA     bool
 }
 
-// MinimumQemuVersion is the minimum supported QEMU version.
-const (
-	MinimumQemuVersion = "4.0.0"
-)
+// minimumQemuVersion returns hardMin and softMin.
+//
+// hardMin is the hard minimum version of QEMU.
+// The driver immediately returns the error when QEMU is older than this version.
+//
+// softMin is the oldest recommended version of QEMU.
+// softMin must be >= hardMin.
+//
+// When updating this function, make sure to update
+// `website/content/en/docs/config/vmtype.md` too.
+func minimumQemuVersion() (hardMin, softMin semver.Version) {
+	var h, s string
+	switch runtime.GOOS {
+	case "darwin":
+		switch runtime.GOARCH {
+		case "arm64":
+			// https://gitlab.com/qemu-project/qemu/-/issues/1990
+			h, s = "8.2.1", "8.2.1"
+		default:
+			// The code specific to QEMU < 7.0 on macOS (https://github.com/lima-vm/lima/pull/703)
+			// was removed in https://github.com/lima-vm/lima/pull/3491
+			h, s = "7.0.0", "8.2.1"
+		}
+	default:
+		// hardMin: Untested and maybe does not even work.
+		// softMin: Ubuntu 22.04's QEMU. The oldest version that can be easily tested on GitHub Actions.
+		h, s = "4.0.0", "6.2.0"
+	}
+	hardMin, softMin = *semver.New(h), *semver.New(s)
+	if softMin.LessThan(hardMin) {
+		// NOTREACHED
+		logrus.Fatalf("internal error: QEMU: soft minimum version %v must be >= hard minimum version %v",
+			softMin, hardMin)
+	}
+	return hardMin, softMin
+}
 
 // EnsureDisk also ensures the kernel and the initrd.
 func EnsureDisk(ctx context.Context, cfg Config) error {
@@ -128,32 +159,6 @@ func EnsureDisk(ctx context.Context, cfg Config) error {
 	return nil
 }
 
-func CreateDataDisk(dir, format string, size int) error {
-	dataDisk := filepath.Join(dir, filenames.DataDisk)
-	if _, err := os.Stat(dataDisk); err == nil || !errors.Is(err, fs.ErrNotExist) {
-		// datadisk already exists
-		return err
-	}
-
-	args := []string{"create", "-f", format, dataDisk, strconv.Itoa(size)}
-	cmd := exec.Command("qemu-img", args...)
-	if out, err := cmd.CombinedOutput(); err != nil {
-		return fmt.Errorf("failed to run %v: %q: %w", cmd.Args, string(out), err)
-	}
-	return nil
-}
-
-func ResizeDataDisk(dir, format string, size int) error {
-	dataDisk := filepath.Join(dir, filenames.DataDisk)
-
-	args := []string{"resize", "-f", format, dataDisk, strconv.Itoa(size)}
-	cmd := exec.Command("qemu-img", args...)
-	if out, err := cmd.CombinedOutput(); err != nil {
-		return fmt.Errorf("failed to run %v: %q: %w", cmd.Args, string(out), err)
-	}
-	return nil
-}
-
 func newQmpClient(cfg Config) (*qmp.SocketMonitor, error) {
 	qmpSock := filepath.Join(cfg.InstanceDir, filenames.QMPSock)
 	qmpClient, err := qmp.NewSocketMonitor("unix", qmpSock, 5*time.Second)
@@ -314,9 +319,6 @@ type features struct {
 	// e.g. "Available CPUs:\n...\nx86 base...\nx86 host...\n...\n"
 	// Not machine-readable, but checking strings.Contains() should be fine.
 	CPUHelp []byte
-
-	// VersionGEQ7 is true when the QEMU version seems v7.0.0 or later
-	VersionGEQ7 bool
 }
 
 func inspectFeatures(exe, machine string) (*features, error) {
@@ -360,7 +362,6 @@ func inspectFeatures(exe, machine string) (*features, error) {
 			f.MachineHelp = stderr.Bytes()
 		}
 	}
-	f.VersionGEQ7 = strings.Contains(string(f.MachineHelp), "-7.0")
 
 	// Avoid error: "No machine specified, and there is no default"
 	cmd = exec.Command(exe, "-cpu", "help", "-machine", machine)
@@ -378,56 +379,15 @@ func inspectFeatures(exe, machine string) (*features, error) {
 	return &f, nil
 }
 
-// showDarwinARM64HVFQEMU620Warning shows a warning on M1 macOS when QEMU is older than 6.2.0_1.
-//
-// See:
-// - https://gitlab.com/qemu-project/qemu/-/issues/899
-// - https://github.com/Homebrew/homebrew-core/pull/96743
-// - https://github.com/lima-vm/lima/issues/712
-func showDarwinARM64HVFQEMU620Warning(exe, accel string, features *features) {
-	if runtime.GOOS != "darwin" {
-		return
-	}
-	if runtime.GOARCH != "arm64" {
-		return
-	}
-	if accel != "hvf" {
-		return
-	}
-	if features.VersionGEQ7 {
-		return
-	}
-	if exeFull, err := exec.LookPath(exe); err == nil {
-		if exeResolved, err2 := filepath.EvalSymlinks(exeFull); err2 == nil {
-			if strings.Contains(exeResolved, "Cellar/qemu/6.2.0_") {
-				// Homebrew's QEMU 6.2.0_1 or later
-				return
-			}
-		}
-	}
-	w := "This version of QEMU might not be able to boot recent Linux guests on M1 macOS hosts."
-	if _, err := exec.LookPath("brew"); err == nil {
-		w += "Run `brew upgrade` and make sure your QEMU version is 6.2.0_1 or later."
-	} else {
-		w += `Reinstall QEMU with the following commits (included in QEMU 7.0.0):
-- https://github.com/qemu/qemu/commit/ad99f64f "hvf: arm: Use macros for sysreg shift/masking"
-- https://github.com/qemu/qemu/commit/7f6c295c "hvf: arm: Handle unknown ID registers as RES0"
-`
-		w += "See https://github.com/Homebrew/homebrew-core/pull/96743 for the further information."
-	}
-	logrus.Warn(w)
-}
-
 // adjustMemBytesDarwinARM64HVF adjusts the memory to be <= 3 GiB, only when the following conditions are met:
 //
 // - Host OS   <  macOS 12.4
 // - Host Arch == arm64
 // - Accel     == hvf
-// - QEMU      >= 7.0
 //
 // This adjustment is required for avoiding host kernel panic. The issue was fixed in macOS 12.4 Beta 1.
 // See https://github.com/lima-vm/lima/issues/795 https://gitlab.com/qemu-project/qemu/-/issues/903#note_911000975
-func adjustMemBytesDarwinARM64HVF(memBytes int64, accel string, features *features) int64 {
+func adjustMemBytesDarwinARM64HVF(memBytes int64, accel string) int64 {
 	const safeSize = 3 * 1024 * 1024 * 1024 // 3 GiB
 	if memBytes <= safeSize {
 		return memBytes
@@ -441,9 +401,6 @@ func adjustMemBytesDarwinARM64HVF(memBytes int64, accel string, features *featur
 	if accel != "hvf" {
 		return memBytes
 	}
-	if !features.VersionGEQ7 {
-		return memBytes
-	}
 	macOSProductVersion, err := osutil.ProductVersion()
 	if err != nil {
 		logrus.Warn(err)
@@ -452,8 +409,8 @@ func adjustMemBytesDarwinARM64HVF(memBytes int64, accel string, features *featur
 	if !macOSProductVersion.LessThan(*semver.New("12.4.0")) {
 		return memBytes
 	}
-	logrus.Warnf("Reducing the guest memory from %s to %s, to avoid host kernel panic on macOS <= 12.3 with QEMU >= 7.0; "+
-		"Please update macOS to 12.4 or later, or downgrade QEMU to 6.2; "+
+	logrus.Warnf("Reducing the guest memory from %s to %s, to avoid host kernel panic on macOS <= 12.3; "+
+		"Please update macOS to 12.4 or later; "+
 		"See https://github.com/lima-vm/lima/issues/795 for the further background.",
 		units.BytesSize(float64(memBytes)), units.BytesSize(float64(safeSize)))
 	memBytes = safeSize
@@ -498,16 +455,16 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		logrus.WithError(err).Warning("Failed to detect QEMU version")
 	} else {
 		logrus.Debugf("QEMU version %s detected", version.String())
-		if version.LessThan(*semver.New(MinimumQemuVersion)) {
-			logrus.Fatalf("QEMU %v is too old, %v or later required", version, MinimumQemuVersion)
+		hardMin, softMin := minimumQemuVersion()
+		if version.LessThan(hardMin) {
+			logrus.Fatalf("QEMU %v is too old, %v or later required", version, hardMin)
+		}
+		if version.LessThan(softMin) {
+			logrus.Warnf("QEMU %v is too old, %v or later is recommended", version, softMin)
 		}
 		if y.VMOpts.QEMU.MinimumVersion != nil && version.LessThan(*semver.New(*y.VMOpts.QEMU.MinimumVersion)) {
 			logrus.Fatalf("QEMU %v is too old, template requires %q or later", version, *y.VMOpts.QEMU.MinimumVersion)
 		}
-		if runtime.GOOS == "darwin" && runtime.GOARCH == "arm64" && version.Equal(*semver.New("8.2.0")) {
-			logrus.Fatal("QEMU 8.2.0 is no longer supported on ARM Mac due to <https://gitlab.com/qemu-project/qemu/-/issues/1990>. " +
-				"Please upgrade QEMU to v8.2.1 (or downgrade to v8.1.x).")
-		}
 	}
 
 	// Architecture
@@ -515,14 +472,13 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	if !strings.Contains(string(features.AccelHelp), accel) {
 		return "", nil, fmt.Errorf("accelerator %q is not supported by %s", accel, exe)
 	}
-	showDarwinARM64HVFQEMU620Warning(exe, accel, features)
 
 	// Memory
 	memBytes, err := units.RAMInBytes(*y.Memory)
 	if err != nil {
 		return "", nil, err
 	}
-	memBytes = adjustMemBytesDarwinARM64HVF(memBytes, accel, features)
+	memBytes = adjustMemBytesDarwinARM64HVF(memBytes, accel)
 	args = appendArgsIfNoConflict(args, "-m", strconv.Itoa(int(memBytes>>20)))
 
 	if *y.MountType == limayaml.VIRTIOFS {
@@ -541,7 +497,8 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 			}
 		}
 	}
-	if !strings.Contains(string(features.CPUHelp), strings.Split(cpu, ",")[0]) {
+	// `qemu-system-ppc64 -help` does not show "max", but it is actually accepted
+	if cpu != "max" && !strings.Contains(string(features.CPUHelp), strings.Split(cpu, ",")[0]) {
 		return "", nil, fmt.Errorf("cpu %q is not supported by %s", cpu, exe)
 	}
 	args = appendArgsIfNoConflict(args, "-cpu", cpu)
@@ -549,7 +506,8 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	// Machine
 	switch *y.Arch {
 	case limayaml.X8664:
-		if strings.HasPrefix(cpu, "qemu64") && runtime.GOOS != "windows" {
+		switch accel {
+		case "tcg":
 			// use q35 machine with vmware io port disabled.
 			args = appendArgsIfNoConflict(args, "-machine", "q35,vmport=off")
 			// use tcg accelerator with multi threading with 512MB translation block size
@@ -560,22 +518,14 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 			// This will disable CPU S3/S4 state.
 			args = append(args, "-global", "ICH9-LPC.disable_s3=1")
 			args = append(args, "-global", "ICH9-LPC.disable_s4=1")
-		} else if runtime.GOOS == "windows" && accel == "whpx" {
+		case "whpx":
 			// whpx: injection failed, MSI (0, 0) delivery: 0, dest_mode: 0, trigger mode: 0, vector: 0
 			args = appendArgsIfNoConflict(args, "-machine", "q35,accel="+accel+",kernel-irqchip=off")
-		} else {
+		default:
 			args = appendArgsIfNoConflict(args, "-machine", "q35,accel="+accel)
 		}
 	case limayaml.AARCH64:
 		machine := "virt,accel=" + accel
-		// QEMU >= 7.0 requires highmem=off NOT to be set, otherwise fails with "Addressing limited to 32 bits, but memory exceeds it by 1073741824 bytes"
-		// QEMU <  7.0 requires highmem=off to be set, otherwise fails with "VCPU supports less PA bits (36) than requested by the memory map (40)"
-		// https://github.com/lima-vm/lima/issues/680
-		// https://github.com/lima-vm/lima/pull/24
-		// But when the memory size is <= 3 GiB, we can always set highmem=off.
-		if !features.VersionGEQ7 || memBytes <= 3*1024*1024*1024 {
-			machine += ",highmem=off"
-		}
 		args = appendArgsIfNoConflict(args, "-machine", machine)
 	case limayaml.RISCV64:
 		// https://github.com/tianocore/edk2/blob/edk2-stable202408/OvmfPkg/RiscVVirt/README.md#test
@@ -588,6 +538,12 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	case limayaml.ARMV7L:
 		machine := "virt,accel=" + accel
 		args = appendArgsIfNoConflict(args, "-machine", machine)
+	case limayaml.PPC64LE:
+		machine := "pseries,accel=" + accel
+		args = appendArgsIfNoConflict(args, "-machine", machine)
+	case limayaml.S390X:
+		machine := "s390-ccw-virtio,accel=" + accel
+		args = appendArgsIfNoConflict(args, "-machine", machine)
 	}
 
 	// SMP
@@ -600,7 +556,8 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		logrus.Warnf("field `firmware.legacyBIOS` is not supported for architecture %q, ignoring", *y.Arch)
 		legacyBIOS = false
 	}
-	if !legacyBIOS {
+	noFirmware := *y.Arch == limayaml.PPC64LE || *y.Arch == limayaml.S390X || legacyBIOS
+	if !noFirmware {
 		var firmware string
 		firmwareInBios := runtime.GOOS == "windows"
 		if envVar := os.Getenv("_LIMA_QEMU_UEFI_IN_BIOS"); envVar != "" {
@@ -783,7 +740,13 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		}
 		args = append(args, "-netdev", fmt.Sprintf("socket,id=net0,fd={{ fd_connect %q }}", qemuSock))
 	}
-	args = append(args, "-device", "virtio-net-pci,netdev=net0,mac="+limayaml.MACAddress(cfg.InstanceDir))
+	virtioNet := "virtio-net-pci"
+	if *y.Arch == limayaml.S390X {
+		// virtio-net-pci does not work on EL, while it works on Ubuntu
+		// https://github.com/lima-vm/lima/pull/3319/files#r1986388345
+		virtioNet = "virtio-net-ccw"
+	}
+	args = append(args, "-device", virtioNet+",netdev=net0,mac="+limayaml.MACAddress(cfg.InstanceDir))
 
 	for i, nw := range y.Networks {
 		if nw.Lima != "" {
@@ -806,7 +769,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 					return "", nil, err
 				}
 				args = append(args, "-netdev", fmt.Sprintf("socket,id=net%d,fd={{ fd_connect %q }}", i+1, qemuSock))
-				args = append(args, "-device", fmt.Sprintf("virtio-net-pci,netdev=net%d,mac=%s", i+1, nw.MACAddress))
+				args = append(args, "-device", fmt.Sprintf("%s,netdev=net%d,mac=%s", virtioNet, i+1, nw.MACAddress))
 			} else {
 				if runtime.GOOS != "darwin" {
 					return "", nil, fmt.Errorf("networks.yaml '%s' configuration is only supported on macOS right now", nw.Lima)
@@ -825,7 +788,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		} else {
 			return "", nil, fmt.Errorf("invalid network spec %+v", nw)
 		}
-		args = append(args, "-device", fmt.Sprintf("virtio-net-pci,netdev=net%d,mac=%s", i+1, nw.MACAddress))
+		args = append(args, "-device", fmt.Sprintf("%s,netdev=net%d,mac=%s", virtioNet, i+1, nw.MACAddress))
 	}
 
 	// virtio-rng-pci accelerates starting up the OS, according to https://wiki.gentoo.org/wiki/QEMU/Options
@@ -862,23 +825,15 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	}
 
 	switch *y.Arch {
+	// FIXME: use virtio-gpu on all the architectures
 	case limayaml.X8664, limayaml.RISCV64:
 		args = append(args, "-device", "virtio-vga")
-		args = append(args, "-device", "virtio-keyboard-pci")
-		args = append(args, "-device", "virtio-"+input+"-pci")
-		args = append(args, "-device", "qemu-xhci,id=usb-bus")
-	case limayaml.AARCH64, limayaml.ARMV7L:
-		if features.VersionGEQ7 {
-			args = append(args, "-device", "virtio-gpu")
-			args = append(args, "-device", "virtio-keyboard-pci")
-			args = append(args, "-device", "virtio-"+input+"-pci")
-		} else { // kernel panic with virtio and old versions of QEMU
-			args = append(args, "-vga", "none", "-device", "ramfb")
-			args = append(args, "-device", "usb-kbd,bus=usb-bus")
-			args = append(args, "-device", "usb-"+input+",bus=usb-bus")
-		}
-		args = append(args, "-device", "qemu-xhci,id=usb-bus")
+	default:
+		args = append(args, "-device", "virtio-gpu")
 	}
+	args = append(args, "-device", "virtio-keyboard-pci")
+	args = append(args, "-device", "virtio-"+input+"-pci")
+	args = append(args, "-device", "qemu-xhci,id=usb-bus")
 
 	// Parallel
 	args = append(args, "-parallel", "none")
@@ -927,7 +882,11 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	const serialvChardev = "char-serial-virtio"
 	args = append(args, "-chardev", fmt.Sprintf("socket,id=%s,path=%s,server=on,wait=off,logfile=%s", serialvChardev, serialvSock, serialvLog))
 	// max_ports=1 is required for https://github.com/lima-vm/lima/issues/1689 https://github.com/lima-vm/lima/issues/1691
-	args = append(args, "-device", "virtio-serial-pci,id=virtio-serial0,max_ports=1")
+	serialvMaxPorts := 1
+	if *y.Arch == limayaml.S390X {
+		serialvMaxPorts++ // needed to avoid `virtio-serial-bus: Out-of-range port id specified, max. allowed: 0`
+	}
+	args = append(args, "-device", fmt.Sprintf("virtio-serial-pci,id=virtio-serial0,max_ports=%d", serialvMaxPorts))
 	args = append(args, "-device", fmt.Sprintf("virtconsole,chardev=%s,id=console0", serialvChardev))
 
 	// We also want to enable vsock here, but QEMU does not support vsock for macOS hosts
@@ -935,11 +894,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	if *y.MountType == limayaml.NINEP || *y.MountType == limayaml.VIRTIOFS {
 		for i, f := range y.Mounts {
 			tag := fmt.Sprintf("mount%d", i)
-			location, err := localpathutil.Expand(f.Location)
-			if err != nil {
-				return "", nil, err
-			}
-			if err := os.MkdirAll(location, 0o755); err != nil {
+			if err := os.MkdirAll(f.Location, 0o755); err != nil {
 				return "", nil, err
 			}
 
@@ -947,7 +902,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 			case limayaml.NINEP:
 				options := "local"
 				options += fmt.Sprintf(",mount_tag=%s", tag)
-				options += fmt.Sprintf(",path=%s", location)
+				options += fmt.Sprintf(",path=%s", f.Location)
 				options += fmt.Sprintf(",security_model=%s", *f.NineP.SecurityModel)
 				if !*f.Writable {
 					options += ",readonly"
@@ -978,11 +933,13 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	args = append(args, "-chardev", fmt.Sprintf("socket,id=%s,path=%s,server=on,wait=off", qmpChardev, qmpSock))
 	args = append(args, "-qmp", "chardev:"+qmpChardev)
 
-	// Guest agent via serialport
-	guestSock := filepath.Join(cfg.InstanceDir, filenames.GuestAgentSock)
-	args = append(args, "-chardev", fmt.Sprintf("socket,path=%s,server=on,wait=off,id=qga0", guestSock))
-	args = append(args, "-device", "virtio-serial")
-	args = append(args, "-device", "virtserialport,chardev=qga0,name="+filenames.VirtioPort)
+	if cfg.VirtioGA {
+		// Guest agent via serialport
+		guestSock := filepath.Join(cfg.InstanceDir, filenames.GuestAgentSock)
+		args = append(args, "-chardev", fmt.Sprintf("socket,path=%s,server=on,wait=off,id=qga0", guestSock))
+		args = append(args, "-device", "virtio-serial")
+		args = append(args, "-device", "virtserialport,chardev=qga0,name="+filenames.VirtioPort)
+	}
 
 	// QEMU process
 	args = append(args, "-name", "lima-"+cfg.Name)
@@ -997,16 +954,16 @@ func FindVirtiofsd(qemuExe string) (string, error) {
 		Binary      string `json:"binary"`
 	}
 
-	currentUser, err := user.Current()
+	homeDir, err := os.UserHomeDir()
 	if err != nil {
 		return "", err
 	}
 
 	const relativePath = "share/qemu/vhost-user"
 
-	binDir := filepath.Dir(qemuExe)                              // "/usr/local/bin"
-	usrDir := filepath.Dir(binDir)                               // "/usr/local"
-	userLocalDir := filepath.Join(currentUser.HomeDir, ".local") // "$HOME/.local"
+	binDir := filepath.Dir(qemuExe)                  // "/usr/local/bin"
+	usrDir := filepath.Dir(binDir)                   // "/usr/local"
+	userLocalDir := filepath.Join(homeDir, ".local") // "$HOME/.local"
 
 	candidates := []string{
 		filepath.Join(userLocalDir, relativePath),
@@ -1067,10 +1024,6 @@ func FindVirtiofsd(qemuExe string) (string, error) {
 
 func VirtiofsdCmdline(cfg Config, mountIndex int) ([]string, error) {
 	mount := cfg.LimaYAML.Mounts[mountIndex]
-	location, err := localpathutil.Expand(mount.Location)
-	if err != nil {
-		return nil, err
-	}
 
 	vhostSock := filepath.Join(cfg.InstanceDir, fmt.Sprintf(filenames.VhostSock, mountIndex))
 	// qemu_driver has to wait for the socket to appear, so make sure any old ones are removed here.
@@ -1080,16 +1033,20 @@ func VirtiofsdCmdline(cfg Config, mountIndex int) ([]string, error) {
 
 	return []string{
 		"--socket-path", vhostSock,
-		"--shared-dir", location,
+		"--shared-dir", mount.Location,
 	}, nil
 }
 
 // qemuArch returns the arch string used by qemu.
 func qemuArch(arch limayaml.Arch) string {
-	if arch == limayaml.ARMV7L {
+	switch arch {
+	case limayaml.ARMV7L:
 		return "arm"
+	case limayaml.PPC64LE:
+		return "ppc64"
+	default:
+		return arch
 	}
-	return arch
 }
 
 // qemuEdk2 returns the arch string used by `/usr/local/share/qemu/edk2-*-code.fd`.
@@ -1129,6 +1086,8 @@ func Accel(arch limayaml.Arch) string {
 			return "kvm"
 		case "netbsd":
 			return "nvmm"
+		case "dragonfly":
+			return "nvmm"
 		case "windows":
 			return "whpx"
 		}
@@ -1168,14 +1127,14 @@ func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 		return "", fmt.Errorf("unexpected architecture: %q", arch)
 	}
 
-	currentUser, err := user.Current()
+	homeDir, err := os.UserHomeDir()
 	if err != nil {
 		return "", err
 	}
 
-	binDir := filepath.Dir(qemuExe)                              // "/usr/local/bin"
-	localDir := filepath.Dir(binDir)                             // "/usr/local"
-	userLocalDir := filepath.Join(currentUser.HomeDir, ".local") // "$HOME/.local"
+	binDir := filepath.Dir(qemuExe)                  // "/usr/local/bin"
+	localDir := filepath.Dir(binDir)                 // "/usr/local"
+	userLocalDir := filepath.Join(homeDir, ".local") // "$HOME/.local"
 
 	relativePath := fmt.Sprintf("share/qemu/edk2-%s-code.fd", qemuEdk2Arch(arch))
 	relativePathWin := fmt.Sprintf("share/edk2-%s-code.fd", qemuEdk2Arch(arch))
@@ -1207,7 +1166,10 @@ func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 		// Fedora package "edk2-arm"
 		candidates = append(candidates, "/usr/share/AAVMF/AAVMF32_CODE.fd")
 	case limayaml.RISCV64:
-		// NOP, as EDK2 for RISCV64 is not packaged yet in well-known distros.
+		// Debian package "qemu-efi-riscv64"
+		candidates = append(candidates, "/usr/share/qemu-efi-riscv64/RISCV_VIRT_CODE.fd")
+		// Fedora package "edk2-riscv64"
+		candidates = append(candidates, "/usr/share/edk2/riscv/RISCV_VIRT_CODE.fd")
 	}
 
 	logrus.Debugf("firmware candidates = %v", candidates)
@@ -1234,14 +1196,14 @@ func getFirmwareVars(qemuExe string, arch limayaml.Arch) (string, error) {
 		return "", fmt.Errorf("unexpected architecture: %q", arch)
 	}
 
-	currentUser, err := user.Current()
+	homeDir, err := os.UserHomeDir()
 	if err != nil {
 		return "", err
 	}
 
-	binDir := filepath.Dir(qemuExe)                              // "/usr/local/bin"
-	localDir := filepath.Dir(binDir)                             // "/usr/local"
-	userLocalDir := filepath.Join(currentUser.HomeDir, ".local") // "$HOME/.local"
+	binDir := filepath.Dir(qemuExe)                  // "/usr/local/bin"
+	localDir := filepath.Dir(binDir)                 // "/usr/local"
+	userLocalDir := filepath.Join(homeDir, ".local") // "$HOME/.local"
 
 	relativePath := fmt.Sprintf("share/qemu/edk2-%s-vars.fd", qemuEdk2Arch(targetArch))
 	relativePathWin := fmt.Sprintf("share/edk2-%s-vars.fd", qemuEdk2Arch(targetArch))
