@@ -21,11 +21,15 @@ import (
 	"text/template"
 	"time"
 
+	"github.com/coreos/go-semver/semver"
 	"github.com/digitalocean/go-qemu/qmp"
 	"github.com/digitalocean/go-qemu/qmp/raw"
 	"github.com/lima-vm/lima/pkg/driver"
+	"github.com/lima-vm/lima/pkg/executil"
 	"github.com/lima-vm/lima/pkg/limayaml"
 	"github.com/lima-vm/lima/pkg/networks/usernet"
+	"github.com/lima-vm/lima/pkg/osutil"
+	"github.com/lima-vm/lima/pkg/qemu/entitlementutil"
 	"github.com/lima-vm/lima/pkg/store"
 	"github.com/lima-vm/lima/pkg/store/filenames"
 	"github.com/sirupsen/logrus"
@@ -40,12 +44,25 @@ type LimaQemuDriver struct {
 }
 
 func New(driver *driver.BaseDriver) *LimaQemuDriver {
+	driver.VSockPort = 0
+	driver.VirtioPort = filenames.VirtioPort
+	// virtserialport doesn't seem to work reliably: https://github.com/lima-vm/lima/issues/2064
+	// but on Windows default Unix socket forwarding is not available
+	if runtime.GOOS != "windows" {
+		driver.VirtioPort = ""
+	}
 	return &LimaQemuDriver{
 		BaseDriver: driver,
 	}
 }
 
 func (l *LimaQemuDriver) Validate() error {
+	if runtime.GOOS == "darwin" {
+		if err := l.checkBinarySignature(); err != nil {
+			return err
+		}
+	}
+
 	if *l.Instance.Config.MountType == limayaml.VIRTIOFS && runtime.GOOS != "linux" {
 		return fmt.Errorf("field `mountType` must be %q or %q for QEMU driver on non-Linux, got %q",
 			limayaml.REVSSHFS, limayaml.NINEP, *l.Instance.Config.MountType)
@@ -76,6 +93,7 @@ func (l *LimaQemuDriver) Start(ctx context.Context) (chan error, error) {
 		LimaYAML:     l.Instance.Config,
 		SSHLocalPort: l.SSHLocalPort,
 		SSHAddress:   l.Instance.SSHAddress,
+		VirtioGA:     l.VirtioPort != "",
 	}
 	qExe, qArgs, err := Cmdline(ctx, qCfg)
 	if err != nil {
@@ -110,6 +128,7 @@ func (l *LimaQemuDriver) Start(ctx context.Context) (chan error, error) {
 	}
 	qCmd := exec.CommandContext(ctx, qExe, qArgsFinal...)
 	qCmd.ExtraFiles = append(qCmd.ExtraFiles, applier.files...)
+	qCmd.SysProcAttr = executil.BackgroundSysProcAttr
 	qStdout, err := qCmd.StdoutPipe()
 	if err != nil {
 		return nil, err
@@ -147,7 +166,7 @@ func (l *LimaQemuDriver) Start(ctx context.Context) (chan error, error) {
 
 		vhostSock := filepath.Join(l.Instance.Dir, fmt.Sprintf(filenames.VhostSock, i))
 		vhostSockExists := false
-		for attempt := 0; attempt < 5; attempt++ {
+		for attempt := range 5 {
 			logrus.Debugf("Try waiting for %s to appear (attempt %d)", vhostSock, attempt)
 
 			if _, err := os.Stat(vhostSock); err != nil {
@@ -231,6 +250,27 @@ func waitFileExists(path string, timeout time.Duration) error {
 	return nil
 }
 
+// Ask the user to sign the qemu binary with the "com.apple.security.hypervisor" if needed.
+// Workaround for https://github.com/lima-vm/lima/issues/1742
+func (l *LimaQemuDriver) checkBinarySignature() error {
+	macOSProductVersion, err := osutil.ProductVersion()
+	if err != nil {
+		return err
+	}
+	// The codesign --xml option is only available on macOS Monterey and later
+	if !macOSProductVersion.LessThan(*semver.New("12.0.0")) {
+		qExe, _, err := Exe(l.BaseDriver.Instance.Arch)
+		if err != nil {
+			return fmt.Errorf("failed to find the QEMU binary for the architecture %q: %w", l.BaseDriver.Instance.Arch, err)
+		}
+		if accel := Accel(l.BaseDriver.Instance.Arch); accel == "hvf" {
+			entitlementutil.AskToSignIfNotSignedProperly(qExe)
+		}
+	}
+
+	return nil
+}
+
 func (l *LimaQemuDriver) changeVNCPassword(password string) error {
 	qmpSockPath := filepath.Join(l.Instance.Dir, filenames.QMPSock)
 	err := waitFileExists(qmpSockPath, 30*time.Second)
